function writeAMC(skel,mot,filename)
% writeAMC(skel,mot,filename)

%SCALE BACK TO INCHES FOR FILE WRITING
mot.rootTranslation=mot.rootTranslation/2.54;

cl1 = clock;

%%%%%%%%%%%%%% applies to BVH-style input files only
if (strcmpi(skel.fileType,'BVH')) % convert BVH Euler angles into ASF-style Euler angles by reversing rotation order
    for k = skel.animated'
        mot.rotationEuler{k} = flipud(mot.rotationEuler{k});
    end
end

%%%%%%%%%%%%%% open AMC file
fid = fopen(filename,'wt');
if fid ~= -1
    %%%%%%% write comment
    clk = fix(clock);
    fprintf(fid,'# AMC file generated by Tido''s AMC writer for MATLAB on %s at %.2d:%.2d:%.2d\n',date,clk(4:end));
    fprintf(fid,'# Corresponding ASF file: %s\n',skel.filename);
    fprintf(fid,'# Number of frames: %d\n',mot.nframes);
    fprintf(fid,'# Sampling rate: %d\n',mot.samplingRate);
    fprintf(fid,':fully-specified\n');
    fprintf(fid,':SAMPLES-PER-SECOND %d\n',mot.samplingRate);
    fprintf(fid,':degrees\n');
    
    nodes = skel.nodes(skel.animated);
    if ~strcmpi(nodes(1).boneName,'root')
        warning(['Bone 1 (' nodes(1).boneName ') renamed to "root"!']);
        nodes(1).boneName = 'root';
    end

    strlen = 0;
    disp('Writing AMC frame number: ');
    for k = 1:mot.nframes
        fprintf(fid,'%d\n',k);
        if (rem(k,10)==0)
            s = [num2str(k) '/' num2str(mot.nframes)];
            disp(char(8*ones(1,strlen+2)));
            strlen = length(s);
            disp(s);
        end
        
% write root DOFs
        fprintf(fid,'root ');
        rotDOFIndex = 1;
        transDOFIndex = 1;
        for j = 1:size(nodes(1).DOF,1)
            if (isempty(nodes(1).DOF{1}))
                break;
            end
            switch (lower(nodes(1).DOF{j}(1)))
                case 'r'
                    fprintf(fid, '%.20g ', mot.rotationEuler{1}(rotDOFIndex,k));
                    rotDOFIndex = rotDOFIndex + 1;
                case 't'
                    fprintf(fid, '%.20g ', mot.rootTranslation(transDOFIndex,k)*skel.lengthUnit);
                    transDOFIndex = transDOFIndex + 1;
                otherwise
                    error(['Invalid DOF specification "' nodes(1).DOF{j} '" in root!']);
            end
        end
        fprintf(fid,'\n');
% write remaining nodes' DOFs
        for j = 2:length(nodes)
            fprintf(fid,'%s ',nodes(j).boneName);
            fprintf(fid,'%.20g ',mot.rotationEuler{nodes(j).ID}(:,k));
            fprintf(fid,'\n');
        end
    end
    s = [num2str(k) '/' num2str(mot.nframes)];
    disp(char(8*ones(1,strlen+2)));
    disp(s);

    fclose(fid);        
else
    error('Could not open AMC file.');
end

t=etime(clock,cl1);
disp(['Wrote AMC file ' filename ' in ' num2str(t) ' seconds.']);


